@@ -24,16 +24,21 @@
 #include <gnome.h>
 #include <glade/glade.h>
 
+#include <gnome-xml/tree.h>
+#include <gnome-xml/parser.h>
+
 #include <signal.h>
-#include <ctype.h>
+#include <dirent.h>
 
 #include "bug-buddy.h"
 #include "util.h"
-#include "distro-redhat.h"
-#include "distro-debian.h"
+#include "distro.h"
+#include "glade-druid.h"
+#include "bts.h"
+#include "ctree-combo.h"
 
 /* libglade callbacks */
-gboolean on_nature_page_next  (GtkWidget *, GtkWidget *);
+gboolean on_nature_page_next  (GtkWidget *, GnomeDruid *);
 gboolean on_less_page_prepare (GtkWidget *, GtkWidget *);
 gboolean on_less_page_next    (GtkWidget *, GtkWidget *);
 gboolean on_misc_page_back    (GtkWidget *, GtkWidget *);
@@ -45,23 +50,21 @@ gboolean on_gnome_page_prepare    (GtkWidget *, GtkWidget *);
 gboolean on_contact_page_next     (GtkWidget *, GtkWidget *);
 void on_version_list_select_row   (GtkCList *list, gint row, gint col,
 				   GdkEventButton *event, gpointer udata);
+
 void update_selected_row      (GtkWidget *widget, gpointer data);
 void on_version_apply_clicked (GtkButton *button, gpointer data);
 void on_file_radio_toggled    (GtkWidget *radio, gpointer data);
 gboolean on_action_page_next  (GtkWidget *page, GtkWidget *druid);
-gboolean on_action_page_back  (GtkWidget *page, GtkWidget *druid);
+gboolean on_action_page_back  (GtkWidget *page, GnomeDruid *druid);
 GtkWidget *make_anim (gchar *widget_name, gchar *string1, 
 		      gchar *string2, gint int1, gint int2);
 GtkWidget *make_pixmap_button (gchar *widget_name, gchar *string1, 
 			       gchar *string2, gint int1, gint int2);
 void on_newreport_radio_toggled (GtkWidget *w, gpointer data);
 void on_existing_radio_toggled (GtkWidget *w, gpointer data);
-
-
-
-#define LINE_WIDTH 72
-
-extern const char *packages[];
+gboolean on_more_page_next (GnomeDruidPage *page, GnomeDruid *druid);
+gboolean on_contact_page_prepare (GnomeDruidPage *page, GnomeDruid *druid);
+gboolean on_version_page_prepare (GnomeDruidPage *page, GnomeDruid *druid);
 
 const gchar *severity[] = { 
 	N_("critical"),
@@ -77,49 +80,10 @@ const gchar *bug_class[][2] = {
 	{ N_("support"),           "support" },
 	{ NULL, NULL } };
 
-struct {
-	/* contact page */
-	gchar *name;
-	gchar *email;
-	
-	/* package page */
-	gchar *package;
-	gchar *package_ver;
-	
-	/* dialog page */
-	gchar *app_file;
-	gchar *pid;
-	
-	/* core page */
-	gchar *core_file;
-} popt_data;
+PoptData popt_data;
 
 DruidData druid_data;
 
-static Distribution distros[] = {
-	{ "Slackware", "/etc/slackware-version", &debian_phy },
-	{ "Debian",    "/etc/debian_version",    &debian_phy },
-	{ "Red Hat",   "/etc/redhat-release",    &redhat_phy },
-	{ "SuSE",      "/etc/SuSE-release",      &redhat_phy },
-	{ "Mandrake",  "/etc/mandrake-release",  &redhat_phy },
-	{ NULL }
-};
-
-static Package package[] = {
-	{ N_("System"),    "uname -a" },
-	{ N_("C library"),  NULL,                    "glibc", "libc6", },
-	{ N_("C compiler"), "gcc --version",          NULL,    NULL, "cc -V", },
-	{ N_("glib"),       "glib-config --version",  "glib",  "libglib1.2" },
-	{ N_("GTK+"),       "gtk-config --version",   "gtk+",  "libgtk1.2" },
-	{ N_("ORBit"),      "orbit-config --version", "ORBit", "liborbit0" },
-	{ N_("gnome-libs"), "gnome-config --version", "gnome-libs", "gnome-libs-data" },
-	{ N_("gnome-core"),  "gnome-config --modversion applets "
-	                     "| grep -v gnome-libs "
-	                     "| sed -e 's^applets-^gnome-core ^g'",
-	  "gnome-core", "gnome-core" },
-	{ NULL }
-};
-
 static const struct poptOption options[] = {
 	{ "name",        0, POPT_ARG_STRING, &popt_data.name,        0, N_("Contact's name"),          N_("NAME") },
 	{ "email",       0, POPT_ARG_STRING, &popt_data.email,       0, N_("Contact's email address"), N_("EMAIL") },
@@ -137,10 +101,10 @@ save_entry (const char *name, const char *name2, const char *save_path)
 	GtkWidget *w;
 	gchar *s;
 
-	w = glade_xml_get_widget (druid_data.xml, name);
+	w = GET_WIDGET (name);
 	s = gtk_entry_get_text (GTK_ENTRY (w));
 	gnome_config_set_string (save_path, s);
-	w = glade_xml_get_widget (druid_data.xml, name2);
+	w = GET_WIDGET (name2);
 	if (GNOME_IS_FILE_ENTRY (w))
 		w = gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (w));
 	gnome_entry_prepend_history (GNOME_ENTRY (w), TRUE, s);
@@ -150,12 +114,20 @@ save_entry (const char *name, const char *name2, const char *save_path)
 static void
 save_config ()
 {
+	GtkWidget *w;
+	gboolean b;
 
 	save_entry ("name_entry",   "name_entry2",   "/bug-buddy/last/name");
 	save_entry ("email_entry",  "email_entry2",  "/bug-buddy/last/email");
 	save_entry ("file_entry",   "file_entry2",   "/bug-buddy/last/bugfile");
 	save_entry ("mailer_entry", "mailer_entry2", "/bug-buddy/last/mailer");
-	
+
+	w = gnome_file_entry_gnome_entry (
+		GNOME_FILE_ENTRY (GET_WIDGET ("include_entry2")));
+	gnome_entry_save_history (GNOME_ENTRY (w));
+
+	b = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (SKIP_CONF));
+	gnome_config_set_bool ("/bug-buddy/last/skip_conf", b);
 	gnome_config_set_int ("/bug-buddy/last/submittype", 
 			      druid_data.submit_type);
 
@@ -185,7 +157,9 @@ load_entry (const char *name, const char *name2,
 static void
 load_config ()
 {
+	GtkWidget *w;
 	char *sendmail;
+	gboolean b;
 
 	load_entry ("name_entry", "name_entry2",
 		    "/bug-buddy/last/name", g_get_real_name ());
@@ -196,6 +170,10 @@ load_config ()
 	load_entry ("file_entry", "file_entry2",
 		    "/bug-buddy/last/bugfile", NULL);
 
+	w = gnome_file_entry_gnome_entry (
+		GNOME_FILE_ENTRY (GET_WIDGET ("include_entry2")));
+	gnome_entry_load_history (GNOME_ENTRY (w));
+
 	sendmail = gnome_is_program_in_path ("sendmail");
 	if (!sendmail) {
 		if (g_file_exists ("/usr/sbin/sendmail"))
@@ -207,6 +185,8 @@ load_config ()
 		    "/bug-buddy/last/mailer", sendmail);
 	g_free (sendmail);
 
+	b = gnome_config_get_bool ("/bug-buddy/last/skip_conf=0");
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (SKIP_CONF), b);
 	druid_data.submit_type = 
 		gnome_config_get_int ("/bug-buddy/last/submittype");
 }
@@ -246,20 +226,36 @@ update_submit_type (GtkWidget *w, gpointer data)
 {
 	CrashType new_type = GPOINTER_TO_INT (data);
 	
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)))
-		druid_data.submit_type = new_type;
+	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)))
+		return FALSE;
+
+	druid_data.submit_type = new_type;
+
+	if (new_type == SUBMIT_FILE)
+		gtk_widget_show (GET_WIDGET ("file_entry2"));
+	else
+		gtk_widget_hide (GET_WIDGET ("file_entry2"));
 	
 	return FALSE;
 }
 
 gboolean
-on_nature_page_next (GtkWidget *page, GtkWidget *druid)
+on_nature_page_next (GtkWidget *page, GnomeDruid *druid)
 {
 	if (druid_data.crash_type != CRASH_NONE)
 		return FALSE;
 
-	gnome_druid_set_page (GNOME_DRUID (druid),
-			      GNOME_DRUID_PAGE (druid_data.action));
+	gnome_druid_set_page (druid, GNOME_DRUID_PAGE (ACTION_PAGE));
+	return TRUE;
+}
+
+gboolean
+on_more_page_next (GnomeDruidPage *page, GnomeDruid *druid)
+{
+	if (druid_data.severity != SEVERITY_WISHLIST)
+		return FALSE;
+
+	gnome_druid_set_page (druid, GNOME_DRUID_PAGE (ACTION_PAGE));
 	return TRUE;
 }
 
@@ -277,6 +273,13 @@ on_less_page_prepare (GtkWidget *page, GtkWidget *druid)
 	return FALSE;
 }
 
+gboolean
+on_contact_page_prepare (GnomeDruidPage *page, GnomeDruid *druid)
+{
+	gnome_druid_set_buttons_sensitive (druid, FALSE, TRUE, TRUE);	
+	return FALSE;
+}
+
 static void
 on_stop_button_clicked (GtkWidget *button, gpointer data)
 {
@@ -303,13 +306,19 @@ on_stop_button_clicked (GtkWidget *button, gpointer data)
 }
 
 gboolean
-on_action_page_back (GtkWidget *page, GtkWidget *druid)
+on_action_page_back (GtkWidget *page, GnomeDruid *druid)
 {
-	if (druid_data.crash_type != CRASH_NONE)
+	GnomeDruidPage *newpage;
+	if (druid_data.crash_type != CRASH_NONE &&
+	    druid_data.severity != SEVERITY_WISHLIST)
 		return FALSE;
 
-	gnome_druid_set_page (GNOME_DRUID (druid),
-			      GNOME_DRUID_PAGE (druid_data.nature));
+	if (druid_data.crash_type == SEVERITY_WISHLIST)
+		newpage = GNOME_DRUID_PAGE (MORE_PAGE);
+	else
+		newpage = GNOME_DRUID_PAGE (NATURE_PAGE);
+
+	gnome_druid_set_page (druid, newpage);
 	return TRUE;
 }
 
@@ -354,6 +363,39 @@ on_the_druid_cancel (GtkWidget *w)
 	return FALSE;
 }
 
+static gboolean
+on_timer (GtkProgress *progress)
+{
+	static int orient = 0;	
+	gfloat val = gtk_progress_get_value (progress);
+	if (val >= 100.0) {
+		gtk_progress_bar_set_orientation (GTK_PROGRESS_BAR (progress),
+						  (orient+1)%2);
+		val = -5.0;
+	}
+	gtk_progress_set_value (progress, val+5.0);
+	return TRUE;
+}
+
+gboolean
+on_version_page_prepare (GnomeDruidPage *page, GnomeDruid *druid)
+{
+	GtkWidget *w;
+
+	gtk_entry_set_text (GTK_ENTRY (VERSION_EDIT), "");
+
+	w = GET_WIDGET ("config_progress");
+	gtk_progress_bar_set_activity_step (GTK_PROGRESS_BAR (w), 5.0);
+	gtk_progress_set_activity_mode (GTK_PROGRESS (w), TRUE);
+
+	druid_data.progress_timeout = gtk_timeout_add (150, 
+						       (GtkFunction) on_timer,
+						       (gpointer) w);
+	load_bts_xml ();
+
+	return FALSE;
+}
+
 gboolean
 on_contact_page_next (GtkWidget *page, GtkWidget *druid)
 {
@@ -396,13 +438,20 @@ on_contact_page_next (GtkWidget *page, GtkWidget *druid)
 		w = glade_xml_get_widget (druid_data.xml, "file_radio");
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
 					      TRUE);
-		return FALSE;
+	} else {
+		w = glade_xml_get_widget (druid_data.xml, "submit_radio");
+		gtk_widget_show (w);
+		w = glade_xml_get_widget (druid_data.xml, "email_radio");
+		gtk_widget_show (w);
 	}
+#if 0
+	if (load_bts_xml ())
+		return TRUE;
+
+	gnome_druid_set_buttons_sensitive (GNOME_DRUID (druid), 
+					   TRUE, TRUE, TRUE);
+#endif
 
-	w = glade_xml_get_widget (druid_data.xml, "submit_radio");
-	gtk_widget_show (w);
-	w = glade_xml_get_widget (druid_data.xml, "email_radio");
-	gtk_widget_show (w);
 
 	return FALSE;
 
@@ -421,7 +470,7 @@ on_complete_page_prepare (GtkWidget *page, GtkWidget *druid)
 	int bugnum;
 	to = SUBMIT_ADDRESS;
 
-	w = glade_xml_get_widget (druid_data.xml, "email_entry");
+	w = GET_WIDGET ("email_entry");
 	from = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
 
 	to = NULL;
@@ -436,31 +485,35 @@ on_complete_page_prepare (GtkWidget *page, GtkWidget *druid)
 			break;
 		}
 
-		w = glade_xml_get_widget (druid_data.xml, "bug_number");
+		w = GET_WIDGET ("bug_number");
 		bugnum = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (w));
 		to = g_strdup_printf ("%d" SUBMIT_ADDRESS, bugnum);
 		break;
 	case SUBMIT_FILE:
-		w = glade_xml_get_widget (druid_data.xml, "file_entry");
+		w = GET_WIDGET ("file_entry");
 		to = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
 		break;
+#ifdef SUBMIT_NONE
 	case SUBMIT_NONE:
 		gnome_druid_page_finish_set_text (GNOME_DRUID_PAGE_FINISH (page),
 						  _("Click \"Finish\" to exit bug-buddy.\n"
 						    "No bug report will be submitted."));
+		g_free (from);
 		return FALSE;
+#endif
 	default:
 		g_assert_not_reached ();
 		return FALSE;
 	}
 
-	w = glade_xml_get_widget (druid_data.xml, "name_entry");
+	w = GET_WIDGET ("name_entry");
 	name = gtk_entry_get_text (GTK_ENTRY (w));
 
-	w = glade_xml_get_widget (druid_data.xml, "desc_entry");
+	w = GET_WIDGET ("desc_entry");
 	subject = gtk_entry_get_text (GTK_ENTRY (w));
 
-	w = glade_xml_get_widget (druid_data.xml, "package_entry");
+	w = GET_WIDGET ("miggie_combo");
+	w = CTREE_COMBO (w)->entry;
 	package = gtk_entry_get_text (GTK_ENTRY (w));
 	if (!strlen(package))
 		package = "general";
@@ -480,178 +533,20 @@ on_complete_page_prepare (GtkWidget *page, GtkWidget *druid)
 					package, to, name, from, subject);
 
 	gnome_druid_page_finish_set_text (GNOME_DRUID_PAGE_FINISH (page), text);
+	g_free (to);
 	g_free (text);
+	g_free (from);
 	return FALSE;
 }
 
-static void
-write_line_width (FILE *fp, char *s)
-{
-	gchar *sp;
-	if (!s)
-		return;
-
-	if (strlen (s) < LINE_WIDTH) {
-		fprintf (fp, "%s\n", s);
-		return;
-	}
-
-	for (sp = s+LINE_WIDTH; sp > s && !isspace (*sp); sp--)
-		;
-
-	if (s == sp)
-		sp = strpbrk (s+LINE_WIDTH, "\t\n ");
-       
-	if (sp)
-		*sp = '\0';
-
-	fprintf (fp, "%s\n", s);
-
-	if (sp)
-		write_line_width (fp, sp+1);
-}
-
-static void
-write_line_widthv (FILE *fp, const char *s)
-{
-	int i;
-	gchar **sv = g_strsplit (s, "\n", 0);
-	
-	for (i = 0; sv[i]; i++)
-		write_line_width (fp, sv[i]);
-	
-	g_strfreev (sv);
-}
 
 gboolean
 on_complete_page_finish (GtkWidget *page, GtkWidget *druid)
 {
-	GtkWidget *w;
-	gchar *s, *s2, *s3, *subject;
-	char *command;
-	FILE *fp = stdout;
-	int status, bugnum, i;
-
-	if (druid_data.bug_type == BUG_NEW) {
-		s2 = g_strdup ("submit" SUBMIT_ADDRESS);
-	} else {	
-		w = glade_xml_get_widget (druid_data.xml, "bug_number");
-		bugnum = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (w));
-		s2 = g_strdup_printf ("%d" SUBMIT_ADDRESS, bugnum);
-	}
-		
-	w = glade_xml_get_widget (druid_data.xml, "email_entry");
-	s = gtk_entry_get_text (GTK_ENTRY (w));
-
-	switch (druid_data.submit_type) {
-	case SUBMIT_TO_SELF:
-		s2 = s;
-		/* fall through */
-	case SUBMIT_REPORT:
-		w = glade_xml_get_widget (druid_data.xml, "mailer_entry");
-		s3 = gtk_entry_get_text (GTK_ENTRY (w));
-		command = g_strconcat (s3, " -i -t", NULL);
-		g_message (_("about to run '%s'"), command);
-		fp = popen (command, "w");	       
-		if (!fp) {
-			s = g_strdup_printf (_("Unable to start mail program:\n"
-					       "'%s'"), s3);
-			w = gnome_error_dialog (s);
-			g_free (s);
-			g_free (command);
-			gnome_dialog_run_and_close (GNOME_DIALOG (w));
-			return FALSE;
-		}
-		g_free (command);
-		break;
-	case SUBMIT_FILE:
-		w = glade_xml_get_widget (druid_data.xml, "file_entry");
-		s3 = gtk_entry_get_text (GTK_ENTRY (w));
-		fp = fopen (s3, "w");
-		if (!fp) {
-			s = g_strdup_printf (_("Unable to open file:\n"
-						      "'%s'"), s3);
-			w = gnome_error_dialog (s);
-			g_free (s);
-			gnome_dialog_run_and_close (GNOME_DIALOG (w));
-			return FALSE;
-		}
-		break;
-	case SUBMIT_NONE:
-		save_config ();
-		gtk_main_quit ();
-		return FALSE;
-	default:
-		g_assert_not_reached ();
-		return FALSE;
-	}
-
-	fprintf (fp, "To: %s\n", s2);
-
-	w = glade_xml_get_widget (druid_data.xml, "name_entry");
-	s2 = gtk_entry_get_text (GTK_ENTRY (w));
-	fprintf (fp, "From: %s <%s>\n", s2, s);
-
-	w = glade_xml_get_widget (druid_data.xml, "desc_entry");
-	subject = s = gtk_entry_get_text (GTK_ENTRY (w));
-	fprintf (fp, "Subject: %s\nX-Mailer: %s %s\n", s, PACKAGE, VERSION);
-
-	w = glade_xml_get_widget (druid_data.xml, "package_entry");
-	s = gtk_entry_get_text (GTK_ENTRY (w));
-	if (!strlen(s))
-		s = "general";
-	fprintf (fp, 
-		 "\nPackage: %s\n"
-		 "Severity: %s\n", s, severity[druid_data.severity]);
-
-	w = glade_xml_get_widget (druid_data.xml, "version_entry");
-	s = gtk_entry_get_text (GTK_ENTRY (w));
-	fprintf (fp, 
-		 "Version: %s\n\n"
-		 ">Synopsis: %s\n"
-		 ">Class: %s\n", s, subject, bug_class[druid_data.bug_class][1]);
-
-	for (i = 0; i < GTK_CLIST (druid_data.version_list)->rows; i++) {
-		s = s2 = NULL;
-		gtk_clist_get_text (GTK_CLIST (druid_data.version_list),
-				    i, 0, &s);
-		gtk_clist_get_text (GTK_CLIST (druid_data.version_list),
-				    i, 1, &s2);
-		
-		if (s && strlen (s) &&
-		    s2 && strlen (s2))
-			fprintf (fp, "%s: %s\n", s, s2);
-	}
-
-	w = glade_xml_get_widget (druid_data.xml, "desc_area");
-	s = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
-	if (s && strlen (s)) {
-		fprintf (fp, "\n\n>Description:\n");
-		write_line_widthv (fp, s);
-	}
-	g_free (s);
-
-	w = glade_xml_get_widget (druid_data.xml, "repeat_area");
-	s = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
-	if (s && strlen(s)) {
-		fprintf (fp, "\n\n>How-To-Repeat:\n");
-		write_line_widthv (fp, s);
-	}
-	g_free (s);
-
-	w = glade_xml_get_widget (druid_data.xml, "gdb_text");
-	s = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
-	if (s && strlen(s))
-		fprintf (fp, "\n\nDebugging information:\n%s\n", s);
-	g_free (s);
-
-	status = pclose (fp);
-#if 0
-	g_message (_("Subprocess exited with status %d"), status);
-#endif
+	if (druid_data.bts->doit ())
+		return TRUE;
 	save_config ();
 	gtk_main_quit ();
-
 	return FALSE;
 }
 
@@ -662,14 +557,46 @@ on_version_list_select_row (GtkCList *list, gint row, gint col,
 	gchar *s;
 	druid_data.selected_row = row;
 	if (gtk_clist_get_text (list, row, 1, &s))
-		gtk_entry_set_text (GTK_ENTRY (druid_data.version_edit), s);
+		gtk_entry_set_text (GTK_ENTRY (VERSION_EDIT), s);
 	else
-		gtk_editable_delete_text (GTK_EDITABLE (druid_data.version_edit),
+		gtk_editable_delete_text (GTK_EDITABLE (VERSION_EDIT),
 					  0, -1);
 	gtk_clist_get_text (list, row, 0, &s);
-	gtk_label_set_text (GTK_LABEL (druid_data.version_label), _(s));
+	gtk_label_set_text (GTK_LABEL (VERSION_LABEL), _(s));
 }
 
+static void
+add_to_clist (gpointer data, gpointer udata)
+{
+	char *row[3] = { NULL };
+	Package *package = data;
+	GtkWidget *clist = udata;
+
+	row[0] = _(package->name);
+	row[1] = package->version;
+
+	gtk_clist_append (GTK_CLIST (clist), row);
+}
+
+void
+append_packages ()
+{
+	GtkWidget *w;
+	w = VERSION_LIST;
+
+	gtk_clist_freeze (GTK_CLIST (w));
+	g_slist_foreach (druid_data.packages, add_to_clist, w);
+	gtk_clist_thaw (GTK_CLIST (w));
+
+	gtk_timeout_remove (druid_data.progress_timeout);
+
+	w = GET_WIDGET ("skip_conf");
+	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)))
+		return;
+
+	gnome_druid_set_page (GNOME_DRUID (druid_data.the_druid),
+			      GNOME_DRUID_PAGE (GET_WIDGET ("debian_page")));
+}
 void
 update_selected_row (GtkWidget *w, gpointer data)
 {
@@ -678,8 +605,8 @@ update_selected_row (GtkWidget *w, gpointer data)
 	if (druid_data.selected_row == -1)
 		return;
 	row = druid_data.selected_row;
-	s = gtk_entry_get_text (GTK_ENTRY (druid_data.version_edit));
-	gtk_clist_set_text (GTK_CLIST (druid_data.version_list), row, 1, s);
+	s = gtk_entry_get_text (GTK_ENTRY (VERSION_EDIT));
+	gtk_clist_set_text (GTK_CLIST (VERSION_LIST), row, 1, s);
 }
 
 void
@@ -770,6 +697,12 @@ set_bug_class (GtkWidget *w, gpointer data)
 	return FALSE;
 }
 
+static gboolean
+set_project (GtkWidget *w, gpointer data)
+{
+	druid_data.bts_file = data;
+	return FALSE;
+}
 
 GtkWidget *
 make_pixmap_button (gchar *widget_name, gchar *text, 
@@ -789,57 +722,15 @@ GtkWidget *
 make_anim (gchar *widget_name, gchar *imgname, 
 	   gchar *string2, gint size, gint freq)
 {
+	GtkWidget *w = gnome_animator_new_with_size (48, 48);
 	gchar *pixmap;
-	druid_data.gdb_anim = gnome_animator_new_with_size (48, 48);
-	gnome_animator_set_loop_type (GNOME_ANIMATOR (druid_data.gdb_anim),
+	gnome_animator_set_loop_type (GNOME_ANIMATOR (w),
 				      GNOME_ANIMATOR_LOOP_RESTART);
 	pixmap = g_strconcat (BUDDY_DATADIR, imgname, NULL);
-	gnome_animator_append_frames_from_file (GNOME_ANIMATOR (druid_data.gdb_anim),
+	gnome_animator_append_frames_from_file (GNOME_ANIMATOR (w),
 						pixmap, 0, 0, freq, size);
 	g_free (pixmap);
-	return druid_data.gdb_anim;
-}
-
-/* ugly stupid dumb stuff stolen from the bug reporting web page */
-static gchar *
-get_package_from_appname (const char *appname)
-{
-	FILE *fp;
-	gchar *package;
-	char buf[1024];
-	GHashTable *table;
-
-	fp = fopen (BUDDY_DATADIR "/prog.bugmap", "r");
-	if (!fp) {
-		g_warning ("Could not open bugfile '"
-			   BUDDY_DATADIR
-			   "/prog.bugmap' for reading.");
-		return NULL;
-	}
-
-	table = g_hash_table_new (g_str_hash, g_str_equal);
-	g_hash_table_freeze (table);
-
-	while (fgets (buf, 1024, fp)) {
-		package = strchr (buf, ' ');
-		if (!package)
-			continue;
-		*package = '\0';
-
-		g_hash_table_insert (table, 
-				     g_strdup (buf), 
-				     g_strdup (g_strstrip (package+1)));
-	}
-
-	fclose (fp);
-
-	g_hash_table_thaw (table);
-	package = g_strdup (g_hash_table_lookup (table,
-						 g_filename_pointer (appname)));
-	
-	destroy_hash_table (table, TRUE);
-
-	return package;
+	return w;
 }
 
 static void 
@@ -854,12 +745,68 @@ init_toggle (const char *name, int test, int data, GtkSignalFunc func)
 			    GINT_TO_POINTER (data));
 }
 
+static void
+init_bts_menu (void)
+{
+	GtkWidget *w, *m;
+	DIR *dir;
+	gchar *s = NULL, *p;
+	struct dirent *dent;
+	int idx=0, def=0;
+
+	dir = opendir (BUDDY_DATADIR "/xml/");
+	if (!dir) {
+		s = g_strdup_printf (_("Could not open '%s'.\n"
+				       "Please make sure bug-buddy was "
+				       "installed correctly."),
+				     BUDDY_DATADIR "/xml/");
+		w = gnome_error_dialog (s);
+		g_free (s);
+		gnome_dialog_run_and_close (GNOME_DIALOG (w));
+		exit (0);
+	}
+
+	m = gtk_menu_new ();
+	while ((dent = readdir (dir))) {
+		if (dent->d_name[0] == '.')
+			continue;
+
+		p = strrchr (dent->d_name, '.');
+		if (!p || strcmp (p, ".bts")) 
+			continue;
+		
+		/* memleak */
+		s = g_strdup (dent->d_name);
+		*p = '\0';
+		if (!druid_data.bts_file)
+			druid_data.bts_file = s;
+
+		w = gtk_menu_item_new_with_label (dent->d_name);
+		gtk_signal_connect (GTK_OBJECT (w), "activate",
+				    GTK_SIGNAL_FUNC (set_project), s);
+				    
+		gtk_widget_show (w);
+		gtk_menu_append (GTK_MENU (m), w);
+		if (!strcmp (s, "GNOME.bts")) {
+			druid_data.bts_file = s;
+			def = idx;
+		}
+		idx++;
+	}
+	closedir (dir);
+
+	w = GET_WIDGET ("bts_menu");
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (w), m);
+	gtk_option_menu_set_history (GTK_OPTION_MENU (w), def);
+	/* evil hack */
+	/*if (d) GTK_OPTION_MENU (w)->menu_item = d;*/
+}
+
 static void
 init_ui ()
 {
 	GtkWidget *w, *m, *segv, *core;
-	gchar *row[3] = { NULL };
-	gchar *s, *p = NULL;
+	gchar *s;
 	int i;
 
 	glade_xml_signal_autoconnect (druid_data.xml);
@@ -869,10 +816,7 @@ init_ui ()
 	w = glade_xml_get_widget (druid_data.xml, "the_druid");
 	druid_data.the_druid = w;
 
-	w = glade_xml_get_widget (druid_data.xml, "package_entry2");
-	for (i = 0; packages[i]; i++)
-		gnome_entry_prepend_history (GNOME_ENTRY (w), FALSE,
-					     packages[i]);
+	init_bts_menu ();
 
 	m = gtk_menu_new ();
 	for (i = 0; severity[i]; i++) {
@@ -903,8 +847,7 @@ init_ui ()
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (w), m);
 
 	/* dialog crash page */
-	w = druid_data.app_file = 
-		glade_xml_get_widget (druid_data.xml, "app_file");
+	w = APP_FILE;
 	s = popt_data.app_file;
 	if (!s) {
 		s = getenv ("GNOME_CRASHED_APPNAME");
@@ -917,20 +860,7 @@ init_ui ()
 	if (s)
 		gtk_entry_set_text (GTK_ENTRY (w), s);
 
-	p = g_strdup (popt_data.package);
-	if (!p && s)
-		p = get_package_from_appname (s);
-
-	if (!p)
-		p = g_strdup ("general");
-
-	w = glade_xml_get_widget (druid_data.xml, 
-				  "package_entry");
-	gtk_entry_set_text (GTK_ENTRY (w), p);
-	g_free (p);
-
-	w = druid_data.pid =
-		glade_xml_get_widget (druid_data.xml, "crashed_pid");
+	w = CRASHED_PID;
 	s = popt_data.pid;
 	if (!s) {
 		s = getenv ("GNOME_CRASHED_PID");
@@ -946,8 +876,7 @@ init_ui ()
 	}
 
 	/* core crash page */
-	w = druid_data.core_file = 
-		glade_xml_get_widget (druid_data.xml, "core_file");
+	w = CORE_FILE;
 	if (popt_data.core_file) {
 		gtk_entry_set_text (GTK_ENTRY (w), popt_data.core_file);
 		druid_data.crash_type = CRASH_CORE;
@@ -975,8 +904,10 @@ init_ui ()
 	init_toggle ("email_radio", druid_data.submit_type, SUBMIT_TO_SELF,
 		     GTK_SIGNAL_FUNC (update_submit_type));
 
+#ifdef SUBMIT_NONE
 	init_toggle ("noaction_radio", druid_data.submit_type, SUBMIT_NONE,
 		     GTK_SIGNAL_FUNC (update_submit_type));
+#endif
 
 	init_toggle ("file_radio", druid_data.submit_type, SUBMIT_FILE,
 		     GTK_SIGNAL_FUNC (update_submit_type));
@@ -999,83 +930,33 @@ init_ui ()
 		break;
 	}
 
-	/* system config page */
-	druid_data.version_edit =
-		glade_xml_get_widget (druid_data.xml, "version_edit");
-	druid_data.version_label = 
-		glade_xml_get_widget (druid_data.xml, "version_label");
-	druid_data.version_list = w =
-		glade_xml_get_widget (druid_data.xml, "version_list");
-
-	for (i = 0; distros[i].name; i++) {
-		if (!g_file_exists (distros[i].version_file))
-			continue;
-		row[0] = _("Distribution");
-		row[1] = distros[i].phylum->version (&distros[i]);
-		gtk_clist_append (GTK_CLIST (w), row);
-		druid_data.distro = &distros[i];
-		break;
-	}
-
-	/* so we have like 3 passages, but it is better */
-	for (i = 0; package[i].name; i++) {
-		if (!package[i].pre_command)
-			continue;
-		package[i].version = get_line_from_command (package[i].pre_command);
-	}
-
-	if (druid_data.distro)
-		druid_data.distro->phylum->packager (package);
-
-	for (i = 0; package[i].name; i++) {
-		if (!package[i].version &&
-		    package[i].post_command)
-			package[i].version = 
-				get_line_from_command (package[i].post_command);
-		row[0] = _(package[i].name);
-		row[1] = package[i].version;
-		gtk_clist_append (GTK_CLIST (w), row);
-	}
-
 	w = glade_xml_get_widget (druid_data.xml, "about_button");
 	gtk_signal_connect (GTK_OBJECT (w), "clicked",
 			    GTK_SIGNAL_FUNC (on_about_button_clicked),
 			    NULL);
 
 	/* less page */
-	druid_data.stop_button = w = 
-		glade_xml_get_widget (druid_data.xml, "stop_button");
+	w = STOP_BUTTON;
 	gtk_signal_connect (GTK_OBJECT (w), "clicked",
 			    GTK_SIGNAL_FUNC (on_stop_button_clicked),
 			    NULL);
 	gtk_widget_set_sensitive (GTK_WIDGET (w), FALSE);
 
-	druid_data.refresh_button = w =
-		glade_xml_get_widget (druid_data.xml, "refresh_button");
+	w = REFRESH_BUTTON;
 	gtk_signal_connect (GTK_OBJECT (w), "clicked",
 			    GTK_SIGNAL_FUNC (on_refresh_button_clicked),
 			    NULL);
-	
-	/* other stuff */
-	druid_data.gdb_text = glade_xml_get_widget (druid_data.xml, 
-						    "gdb_text");
-	druid_data.nature   = glade_xml_get_widget (druid_data.xml,
-						    "nature_page");
-	druid_data.attach   = glade_xml_get_widget (druid_data.xml,
-						    "attach_page");
-	druid_data.core     = glade_xml_get_widget (druid_data.xml,
-						    "core_page");
-	druid_data.less     = glade_xml_get_widget (druid_data.xml, 
-						    "less_page");
-	druid_data.action   = glade_xml_get_widget (druid_data.xml, 
-						    "action_page");	
+
+	w = GET_WIDGET ("contact_page");
+	gnome_druid_set_page (GNOME_DRUID (druid_data.the_druid), 
+			      GNOME_DRUID_PAGE (w));
 }
 
 int
 main (int argc, char *argv[])
 {
 	GtkWidget *w;
-	gchar *xml_path, *s;
+	char *s;
 
 	memset (&druid_data, 0, sizeof (druid_data));
 	memset (&popt_data,  0, sizeof (popt_data));
@@ -1087,26 +968,19 @@ main (int argc, char *argv[])
 				    options, 0, NULL);
 	glade_gnome_init ();
 
-	xml_path = BUDDY_DATADIR "/bug-buddy.glade";
-	if (!g_file_exists (xml_path)) {
-		s = g_strdup_printf (_("Could not find our glade file '%s'\n"
-				       "Please make sure bug-buddy was "
-				       "installed correctly."), xml_path);
-		w = gnome_error_dialog (s);
-		gnome_dialog_run_and_close (GNOME_DIALOG (w));
-		return 0;
-	}
-	druid_data.xml = glade_xml_new (xml_path , "druid_window");
-
+	druid_data.xml = glade_xml_new (BUDDY_DATADIR "/bug-buddy.glade", 
+					"druid_window");
 	if (!druid_data.xml) {
-		s = g_strdup_printf (_("XML file '%s' could not be loaded"), 
-				     xml_path);
+		s = g_strdup_printf (_("Could not find '%s'.\n"
+				       "Please make sure bug-buddy was "
+				       "installed correctly."),
+				     BUDDY_DATADIR "/bug-buddy.glade");
 		w = gnome_error_dialog (s);
 		gnome_dialog_run_and_close (GNOME_DIALOG (w));
 		return 0;
 	}
 
-	init_ui (druid_data.xml);
+	init_ui ();
 
 	w = glade_xml_get_widget (druid_data.xml, "druid_window");
 	gtk_widget_show (w);
