@@ -25,6 +25,7 @@
 #include <glade/glade.h>
 #include <sys/types.h>
 #include "distro.h"
+#include "bts.h"
 
 #define SUBMIT_ADDRESS "@bugs.gnome.org"
 
@@ -37,7 +38,9 @@ typedef enum {
 typedef enum {
 	SUBMIT_REPORT,
 	SUBMIT_TO_SELF,
-	SUBMIT_NONE,
+#if 0
+	SUBMIT_DONE,
+#endif
 	SUBMIT_FILE
 } SubmitType;
 
@@ -46,55 +49,76 @@ typedef enum {
 	BUG_EXISTING
 } BugType;
 
+typedef enum {
+	SEVERITY_CRITICAL,
+	SEVERITY_GRAVE,
+	SEVERITY_NORMAL,
+	SEVERITY_WISHLIST,
+} SeverityType;
+
+typedef enum {
+	BUG_CLASS_SW,
+	BUG_CLASS_DOC,
+	BUG_CLASS_CHANGE,
+	BUG_CLASS_SUPPORT
+} BugClassType;
+
+typedef struct {
+	/* contact page */
+	gchar *name;
+	gchar *email;
+	
+	/* package page */
+	gchar *package;
+	gchar *package_ver;
+	
+	/* dialog page */
+	gchar *app_file;
+	gchar *pid;
+	
+	/* core page */
+	gchar *core_file;
+} PoptData;
+extern PoptData popt_data;
+
 typedef struct {
 	GtkWidget *the_druid;
+	GladeXML  *xml;
 
-	GtkWidget *action;
-	GtkWidget *attach;
-	GtkWidget *core;
-	GtkWidget *less;
-	GtkWidget *nature;
+	Distribution      *distro;
+	BugTrackingSystem *bts;
+	char              *bts_file;
 
-	GtkWidget *gdb_text;
-	GtkWidget *gdb_anim;
-	GtkWidget *app_file;
-	GtkWidget *pid;
-	GtkWidget *core_file;
+	CrashType  crash_type;
 
-	GtkWidget *version_edit;
-	GtkWidget *version_label;
-	GtkWidget *version_list;
 	int selected_row;
-	Distribution *distro;
+	int progress_timeout;
 
-	GtkWidget *stop_button;
-	GtkWidget *refresh_button;
-
-	CrashType crash_type;
-	SubmitType submit_type;
-	BugType bug_type;
-
-	int severity;
-	int bug_class;
-
-	GladeXML *xml;
-
-	pid_t app_pid;
-
-	pid_t gdb_pid;
+	/* gdb stuff */
+	pid_t       app_pid;
+	pid_t       gdb_pid;
 	GIOChannel *ioc;
-	int fd;
+	int         fd;
+	gboolean    explicit_dirty;
 
-	gboolean explicit_dirty;
+	/* Debian BTS stuff */
+	SubmitType    submit_type;
+	BugType       bug_type; 
+	SeverityType  severity;
+	BugClassType  bug_class;
+	GSList       *packages;
 } DruidData;
-
 extern DruidData druid_data;
 
+extern const gchar *severity[];
+extern const gchar *bug_class[][2];
+
 void get_trace_from_core (const gchar *core_file);
 void get_trace_from_pair (const gchar *app, const gchar *extra);
 void stop_gdb (void);
 void start_gdb (void);
 
+void append_packages (void);
 #endif /* __bug_buddy_h__ */
 
 
